//This file is meant to be edited

#include <ESP32Servo.h>  // older version <ESP32_Servo.h>, newer version same library <ESP32Servo.h>
#include <Adafruit_NeoPixel.h>  //ESP can use a nicer library, however this conforms to our earlier examples
#include "car.h"

#define NEOPIXEL_PIN 23
#define NEOPIXEL_NUMPIXELS 8

namespace car{
  //define the motorLSpeed and motorRSpeed variables as integers (numbers)
  int motorLSpeed;
  int motorRSpeed;

  Servo myservo;
  Adafruit_NeoPixel pixels(NEOPIXEL_NUMPIXELS, NEOPIXEL_PIN, NEO_GRB + NEO_KHZ800);
  void neopixel_nightrider_update(int r, int g, int b);
  int r = 250, g = 202, b = 48;  // RGB code for RAL 1018, Junior IOT yellow is 250, 202, 48

  void setupFunction(){
    // This code runs once at startup
    myservo.attach(16, 500, 2400);
    pinMode(4, OUTPUT);

    pinMode(22, OUTPUT); //Turn on built in led
    
    pixels.begin(); // INITIALIZE NeoPixel strip object (REQUIRED)
  }

  void loopFunction(){
    // This code runs repeatedly after startup

    //Blink NO DELAY for builtin led
    if ((millis()/100)%10 == 1) {
      digitalWrite(22, false); 
    } else {   
      digitalWrite(22, true);
    }

    neopixel_nightrider_update(r, g, b);

    // can add more to this loop 

    // delay helps free up the core
    delay(20);
  }

  void neopixel_nightrider_update(int r, int g, int b) {
    // we will run this loop very frequently without delay --> will use current millis() to calculate status
    unsigned long this_moment = millis();
  
    const int time_each_pixel = 150; // milisec
    const int num_steps = NEOPIXEL_NUMPIXELS * 2 - 2;
    int prev_step = -1;
    int curr_step = (this_moment/time_each_pixel) % num_steps;
  
    if (curr_step != prev_step) {
      pixels.clear(); // Set all pixel colors to 'off'
  
      if (curr_step < NEOPIXEL_NUMPIXELS) {
        // step 0  1  2  3  4  5  6  7   --> go up from led 0 to 7
        int this_pixel=curr_step;
        
        // pixels.Color() takes RGB values, from 0,0,0 up to 255,255,255
        // Here we're using a moderately bright green color:
        pixels.setPixelColor(this_pixel, pixels.Color(r, g, b));
      } else  {
        // step    8  9 10 11 12 13      --> go backwards from led 6 to 1
        int this_pixel=NEOPIXEL_NUMPIXELS * 2 - 2 - curr_step;
        
        // pixels.Color() takes RGB values, from 0,0,0 up to 255,255,255
        // Here we're using a moderately bright green color:
        pixels.setPixelColor(this_pixel, pixels.Color(r, g, b));
      }
      
      pixels.show();   // Send the updated pixel colors to the hardware.
      prev_step = curr_step;
    }  // if a change was needed

  }

  void movement(int throttle, int rotation){
    // This code runs when the (virtual) joystick values change
    // Do not use delays here
    
    //calculate the motor speed
    motorLSpeed = throttle - rotation;
    motorRSpeed = throttle + rotation;
  
    //write the motor trottle values
    writeMotorL(motorLSpeed);
    writeMotorR(motorRSpeed);
  }

  void button1(){
    // This code runs when button 1 on the controller is pressed
    myservo.write(0);
    r = 250, g = 202, b = 48;  // RGB code for RAL 1018, Junior IOT yellow is 250, 202, 48
    digitalWrite(4, true);      // flame thrower on
    Serial.println("button 1");
  }

  void button2(){
    // This code runs when button 2 on the controller is pressed
    myservo.write(60);
    r = 150, g = 0, b = 0; 
    Serial.println("button 2");
  }

  void button3(){
    // This code runs when button 3 on the controller is pressed
    myservo.write(120);
    r = 0, g = 150, b = 0; 
    Serial.println("button 3");
  }

  void button4(){
    // This code runs when button 4 on the controller is pressed
    myservo.write(180);
    r = 0, g = 0, b = 150; 
    digitalWrite(4, false);   // flame thrower off
    Serial.println("button 4");
  }
}
